import torch
import argparse
import time
import torch
import torch.utils.data
import torch.optim as optim
import numpy as np
import math
import random
import torch.utils.data as data_utils

from sklearn.model_selection import StratifiedShuffleSplit
from sklearn import metrics, model_selection
from load_data import load_dataset
import json
import pathlib
import time
from numpy import loadtxt, savetxt
import matplotlib.pyplot as plt

import numpy as np 
import pandas as pd
from pandas.plotting import scatter_matrix
import seaborn as sns
from vae_gumbel_softmax3 import VAE_gumbel, latent_repr, evaluate
from cnn.train import Training


def classification_latent_space(args, model, data_pos, data_neg):
    
    if args.cuda:
        model.cuda()
        print("Model on GPU")
    
    curr_dir_path = str(pathlib.Path().absolute())
    save_dir_path = '/cnn/' + '/logs/'
    
   
    latent_pos = latent_repr(data_pos, model, args)
    latent_neg = latent_repr(data_neg, model, args)
    
    dim = latent_neg.shape[1]
    print("dimension", dim)
    
    
    if args.incl_rec ==1:
        print("Including the reconstruction error as an extra feature")
        loss2 = evaluate(data_pos, model, args) 
        loss3 = evaluate(data_neg, model, args) 
     #   print(loss2)
        loss2 = np.expand_dims(loss2, 1)
        loss3 = np.expand_dims(loss3, 1)
        positives = np.concatenate((latent_pos, loss2), axis=1)
        negatives = np.concatenate((latent_neg, loss3), axis=1)       
        dim += 1
    else:    
        positives = latent_pos
        negatives = latent_neg
        


    y_pos = np.ones((positives.shape[0], 1))
    y_neg = np.zeros((negatives.shape[0], 1))
    
    sequences = np.concatenate((positives, negatives), axis=0)
    labels = np.concatenate((y_pos, y_neg), axis=0)
    print("positives", latent_pos.shape)
    print("negatives", latent_neg.shape)
    
    #Shuffle the data (positive and negative examples should be shuffled)
    N = labels.shape[0]
    indices = np.arange(N)
    np.random.shuffle(indices)
    sequences = sequences[indices, :]
    labels = labels[indices]
    
    print("total dataset size", sequences.shape)
    print(" batch size ", args.batch_size)
    # Stratified splits 
    train_val_data, test_data, train_val_labels, test_labels = model_selection.train_test_split(sequences, labels, test_size=0.05,
                                                                                                        random_state=1, stratify=labels)
    train_data, val_data, train_labels, val_labels = model_selection.train_test_split(train_val_data,train_val_labels,test_size=0.05,
                                                                                                  random_state=1, stratify=train_val_labels)

    train_accuracy_curve = []
    train_f1_curve = []
    val_accuracy_curve = []
    val_f1_curve = []
    
    #train the model 
    print("start training the model in the latent space")
    # Get config file
    with open(curr_dir_path + "/cnn" + "/config.json", encoding='utf-8', errors='ignore') as json_data:
        config = json.load(json_data, strict=False)   

    config['TRAINER']['save_dir'] = save_dir_path
    obj = Training(config, dim, save_dir_path)
    train_accuracy_curve, train_f1_curve, val_accuracy_curve, val_f1_curve = obj.training_pipeline(train_data, train_labels, val_data, val_labels, test_data, test_labels)
    
    #Testing
    metrics = obj.evaluate(1, test_data, test_labels, testing = True)
    
    return train_accuracy_curve, train_f1_curve, val_accuracy_curve, val_f1_curve, obj